@@ -30,6 +30,7 @@
 #include "synthv1_reverb.h"
 
 #include "synthv1_config.h"
+#include "synthv1_controls.h"
 #include "synthv1_programs.h"
 
 
@@ -749,15 +750,15 @@ class synthv1_impl
 {
 public:
 
-	synthv1_impl(synthv1 *pSynth, uint16_t iChannels, uint32_t iSampleRate);
+	synthv1_impl(synthv1 *pSynth, uint16_t nchannels, float srate);
 
 	~synthv1_impl();
 
-	void setChannels(uint16_t iChannels);
+	void setChannels(uint16_t nchannels);
 	uint16_t channels() const;
 
-	void setSampleRate(uint32_t iSampleRate);
-	uint32_t sampleRate() const;
+	void setSampleRate(float srate);
+	float sampleRate() const;
 
 	void setParamPort(synthv1::ParamIndex index, float *pfParam = 0);
 	float *paramPort(synthv1::ParamIndex index) const;
@@ -765,6 +766,7 @@ public:
 	void setParamValue(synthv1::ParamIndex index, float fValue);
 	float paramValue(synthv1::ParamIndex index) const;
 
+	synthv1_controls *controls();
 	synthv1_programs *programs();
 
 	void process_midi(uint8_t *data, uint32_t size);
@@ -817,10 +819,11 @@ protected:
 private:
 
 	synthv1_config   m_config;
+	synthv1_controls m_controls;
 	synthv1_programs m_programs;
 
-	uint16_t m_iChannels;
-	uint32_t m_iSampleRate;
+	uint16_t m_nchannels;
+	float    m_srate;
 
 	synthv1_ctl m_ctl1, m_ctl2;
 
@@ -891,8 +894,8 @@ synthv1_voice::synthv1_voice ( synthv1_impl *pImpl ) :
 // engine constructor
 
 synthv1_impl::synthv1_impl (
-	synthv1 *pSynth, uint16_t iChannels, uint32_t iSampleRate )
-	: m_programs(pSynth)
+	synthv1 *pSynth, uint16_t nchannels, float srate )
+	: m_controls(pSynth), m_programs(pSynth)
 {
 	// max env. stage length (default)
 	m_dco1.envtime0 = m_dco2.envtime0 = 0.0001f * MAX_ENV_MSECS;
@@ -930,18 +933,19 @@ synthv1_impl::synthv1_impl (
 	m_del.bpmsync0 = 0.0f;
 	m_del.bpm0 = 0;
 
-	// load programs database...
+	// load controllers & programs database...
+	m_config.loadControls(&m_controls);
 	m_config.loadPrograms(&m_programs);
 
 	// number of channels
-	setChannels(iChannels);
+	setChannels(nchannels);
 
 	// parameters
 	for (int i = 0; i < int(synthv1::NUM_PARAMS); ++i)
 		setParamPort(synthv1::ParamIndex(i));
 
 	// set default sample rate
-	setSampleRate(iSampleRate);
+	setSampleRate(srate);
 
 	// reset all voices
 	allControllersOff();
@@ -970,9 +974,9 @@ synthv1_impl::~synthv1_impl (void)
 }
 
 
-void synthv1_impl::setChannels ( uint16_t iChannels )
+void synthv1_impl::setChannels ( uint16_t nchannels )
 {
-	m_iChannels = iChannels;
+	m_nchannels = nchannels;
 
 	// deallocate flangers
 	if (m_flanger) {
@@ -1002,38 +1006,38 @@ void synthv1_impl::setChannels ( uint16_t iChannels )
 
 uint16_t synthv1_impl::channels (void) const
 {
-	return m_iChannels;
+	return m_nchannels;
 }
 
 
-void synthv1_impl::setSampleRate ( uint32_t iSampleRate )
+void synthv1_impl::setSampleRate ( float srate )
 {
 	// set internal sample rate
-	m_iSampleRate = iSampleRate;
+	m_srate = srate;
 
 	// update waves sample rate
-	dco1_wave1.setSampleRate(m_iSampleRate);
-	dco1_wave2.setSampleRate(m_iSampleRate);
-	dco2_wave1.setSampleRate(m_iSampleRate);
-	dco2_wave2.setSampleRate(m_iSampleRate);
+	dco1_wave1.setSampleRate(m_srate);
+	dco1_wave2.setSampleRate(m_srate);
+	dco2_wave1.setSampleRate(m_srate);
+	dco2_wave2.setSampleRate(m_srate);
 
-	lfo1_wave.setSampleRate(m_iSampleRate);
-	lfo2_wave.setSampleRate(m_iSampleRate);
+	lfo1_wave.setSampleRate(m_srate);
+	lfo2_wave.setSampleRate(m_srate);
 
 	updateEnvTimes();
 }
 
 
-uint32_t synthv1_impl::sampleRate (void) const
+float synthv1_impl::sampleRate (void) const
 {
-	return m_iSampleRate;
+	return m_srate;
 }
 
 
 void synthv1_impl::updateEnvTimes_1 (void)
 {
 	// update envelope range times in frames
-	const float srate_ms = 0.001f * float(m_iSampleRate);
+	const float srate_ms = 0.001f * m_srate;
 
 	float envtime_msecs = 10000.0f * m_dco1.envtime0;
 	if (envtime_msecs < MIN_ENV_MSECS)
@@ -1056,7 +1060,7 @@ void synthv1_impl::updateEnvTimes_1 (void)
 void synthv1_impl::updateEnvTimes_2 (void)
 {
 	// update envelope range times in frames
-	const float srate_ms = 0.001f * float(m_iSampleRate);
+	const float srate_ms = 0.001f * m_srate;
 
 	float envtime_msecs = 10000.0f * m_dco2.envtime0;
 	if (envtime_msecs < MIN_ENV_MSECS)
@@ -1218,6 +1222,52 @@ void synthv1_impl::setParamPort ( synthv1::ParamIndex index, float *pfParam )
 	case synthv1::DYN1_LIMITER:   m_dyn.limiter      = pfParam; break;
 	default: break;
 	}
+
+	// check null connections.
+	if (pfParam == &s_fDummy)
+		return;
+
+	// reset ramps after port (re)connection.
+	switch (index) {
+	case synthv1::OUT1_VOLUME:
+	case synthv1::DCA1_VOLUME:
+		m_vol1.reset(
+			m_out1.volume,
+			m_dca1.volume,
+			&m_ctl1.volume,
+			&m_aux1.volume);
+		break;
+	case synthv1::OUT1_WIDTH:
+		m_wid1.reset(
+			m_out1.width);
+		break;
+	case synthv1::OUT1_PANNING:
+		m_pan1.reset(
+			m_out1.panning,
+			&m_ctl1.panning,
+			&m_aux1.panning);
+		break;
+	case synthv1::OUT2_VOLUME:
+	case synthv1::DCA2_VOLUME:
+		m_vol2.reset(
+			m_out2.volume,
+			m_dca2.volume,
+			&m_ctl2.volume,
+			&m_aux2.volume);
+		break;
+	case synthv1::OUT2_WIDTH:
+		m_wid2.reset(
+			m_out2.width);
+		break;
+	case synthv1::OUT2_PANNING:
+		m_pan2.reset(
+			m_out2.panning,
+			&m_ctl2.panning,
+			&m_aux2.panning);
+		break;
+	default:
+		break;
+	}
 }
 
 
@@ -1362,7 +1412,7 @@ void synthv1_impl::setParamValue ( synthv1::ParamIndex index, float fValue )
 {
 	float *pfParamPort = paramPort(index);
 	if (pfParamPort)
-		*pfParamPort = fValue;
+		*pfParamPort = fValue; // LV2:BUG?
 }
 
 
@@ -1377,310 +1427,327 @@ float synthv1_impl::paramValue ( synthv1::ParamIndex index ) const
 
 void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 {
-	// check data size (#1)
-	if (size < 2)
-		return;
+	for (uint32_t i = 0; i < size; ++i) {
 
-	// channel filter
-	const int channel = (data[0] & 0x0f) + 1;
+		// channel status
+		const int channel = (data[i] & 0x0f) + 1;
+		const int status  = (data[i] & 0xf0);
 
-	const int ch1 = int(*m_def1.channel);
-	const int ch2 = int(*m_def2.channel);
-	const int on1 = (ch1 == 0 || ch1 == channel);
-	const int on2 = (ch2 == 0 || ch2 == channel);
+		// channel filter
+		const int ch1 = int(*m_def1.channel);
+		const int ch2 = int(*m_def2.channel);
+		const int on1 = (ch1 == 0 || ch1 == channel);
+		const int on2 = (ch2 == 0 || ch2 == channel);
 
-	if (!on1 && !on2)
-		return;
+		// all system common/real-time ignored
+		if (status == 0xf0)
+			continue;
 
-	// note on
-	const int status = (data[0] & 0xf0);
-	const int key    = (data[1] & 0x7f);
+		// check data size (#1)
+		if (++i >= size)
+			break;
+
+		const int key = (data[i] & 0x7f);
+
+		// program change
+		if (status == 0xc0) {
+			if (on1 || on2) m_programs.prog_change(key);
+			continue;
+		}
 
-	// program change
-	if (status == 0xc0)
-		m_programs.prog_change(key);
-	else
-	if (status == 0xd0) {
 		// channel aftertouch
-		const float pre = float(key) / 127.0f;
-		if (on1) m_ctl1.pressure = pre;
-		if (on2) m_ctl2.pressure = pre;
-	}
-
-	// check data size (#2)
-	if (size < 3)
-		return;
-
-	const int value  = (data[2] & 0x7f);
-
-	// note on
-	if (status == 0x90 && value > 0) {
-		synthv1_voice *pv;
-		// synth 1
-		if (on1) {
-			// mono voice modes
-			if (*m_def1.mono > 0.0f) {
-				for (pv = m_play_list.next(); pv; pv = pv->next()) {
-					if (pv->note1 >= 0
-						&& pv->dca1_env.stage != synthv1_env::Release) {
-						m_dcf1.env.note_off_fast(&pv->dcf1_env);
-						m_lfo1.env.note_off_fast(&pv->lfo1_env);
-						m_dca1.env.note_off_fast(&pv->dca1_env);
-						m_note1[pv->note1] = 0;
-						pv->note1 = -1;
-					}
-				}
-			}
-			// note retrigger
-			pv = m_note1[key];
-			if (pv && pv->note1 >= 0/* && !m_ctl1.sustain*/) {
-				// retrigger fast release
-				m_dcf1.env.note_off_fast(&pv->dcf1_env);
-				m_lfo1.env.note_off_fast(&pv->lfo1_env);
-				m_dca1.env.note_off_fast(&pv->dca1_env);
-				m_note1[pv->note1] = 0;
-				pv->note1 = -1;
-			}
+		if (status == 0xd0) {
+			const float pre = float(key) / 127.0f;
+			if (on1) m_ctl1.pressure = pre;
+			if (on2) m_ctl2.pressure = pre;
+			continue;
 		}
-		// synth 2
-		if (on2) {
-			// mono voice modes
-			if (*m_def2.mono > 0.0f) {
-				for (pv = m_play_list.next(); pv; pv = pv->next()) {
-					if (pv->note2 >= 0
-						&& pv->dca2_env.stage != synthv1_env::Release) {
-						m_dcf2.env.note_off_fast(&pv->dcf2_env);
-						m_lfo2.env.note_off_fast(&pv->lfo2_env);
-						m_dca2.env.note_off_fast(&pv->dca2_env);
-						m_note2[pv->note2] = 0;
-						pv->note2 = -1;
-					}
-				}
-			}
-			// note retrigger
-			pv = m_note2[key];
-			if (pv && pv->note2 >= 0/* && !m_ctl2.sustain*/) {
-				// retrigger fast release
-				m_dcf2.env.note_off_fast(&pv->dcf2_env);
-				m_lfo2.env.note_off_fast(&pv->lfo2_env);
-				m_dca2.env.note_off_fast(&pv->dca2_env);
-				m_note2[pv->note2] = 0;
-				pv->note2 = -1;
-			}
-		}
-		// find free voice
-		pv = alloc_voice();
-		if (pv) {
-			const float srate = float(m_iSampleRate);
-			// velocity (quadratic velocity law)
-			float vel = float(value) / 127.0f; vel *= vel;
+
+		// check data size (#2)
+		if (++i >= size)
+			break;
+
+		// channel filter
+		if (!on1 && !on2)
+			continue;
+
+		const int value = (data[i] & 0x7f);
+
+		// note on
+		if (status == 0x90 && value > 0) {
+			synthv1_voice *pv;
 			// synth 1
 			if (on1) {
-				// waveform
-				pv->note1 = key;
-				pv->vel1 = synthv1_velocity(vel, *m_def1.velocity);
-				// balance
-				pv->dco1_bal.reset(m_dco1.balance);
-				// pressure/after-touch
-				pv->pre1 = 0.0f;
-				pv->dca1_pre.reset(m_def1.pressure, &m_ctl1.pressure, &pv->pre1);
-				// frequencies
-				const float note1 = float(key)
-					+ *m_dco1.octave * OCTAVE_SCALE
-					+ *m_dco1.tuning * TUNING_SCALE;
-				const float detune1
-					= *m_dco1.detune * DETUNE_SCALE;
-				pv->dco1_freq1 = synthv1_freq(note1 - detune1);
-				pv->dco1_freq2 = synthv1_freq(note1 + detune1);
-				// phases
-				const float phase1 = *m_dco1.phase * PHASE_SCALE;
-				pv->dco1_sample1 = pv->dco1_osc1.start(  0.0f, pv->dco1_freq1);
-				pv->dco1_sample2 = pv->dco1_osc2.start(phase1, pv->dco1_freq2);
-				// filters
-				const int type1 = int(*m_dcf1.type);
-				pv->dcf11.reset(synthv1_filter1::Type(type1));
-				pv->dcf12.reset(synthv1_filter1::Type(type1));
-				pv->dcf13.reset(synthv1_filter2::Type(type1));
-				pv->dcf14.reset(synthv1_filter2::Type(type1));
-				// envelopes
-				m_dcf1.env.start(&pv->dcf1_env);
-				m_lfo1.env.start(&pv->lfo1_env);
-				m_dca1.env.start(&pv->dca1_env);
-				// lfos
-				pv->lfo1_sample = pv->lfo1_osc.start();
-				// glides (portamento)
-				const float frames1
-					= uint32_t(*m_dco1.glide * *m_dco1.glide * srate);
-				pv->dco1_glide1.reset(frames1, pv->dco1_freq1);
-				pv->dco1_glide2.reset(frames1, pv->dco1_freq2);
-				// sustain
-				pv->sustain1 = false;
-				// allocated
-				m_note1[key] = pv;
+				// mono voice modes
+				if (*m_def1.mono > 0.0f) {
+					for (pv = m_play_list.next(); pv; pv = pv->next()) {
+						if (pv->note1 >= 0
+							&& pv->dca1_env.stage != synthv1_env::Release) {
+							m_dcf1.env.note_off_fast(&pv->dcf1_env);
+							m_lfo1.env.note_off_fast(&pv->lfo1_env);
+							m_dca1.env.note_off_fast(&pv->dca1_env);
+							m_note1[pv->note1] = 0;
+							pv->note1 = -1;
+						}
+					}
+				}
+				// note retrigger
+				pv = m_note1[key];
+				if (pv && pv->note1 >= 0/* && !m_ctl1.sustain*/) {
+					// retrigger fast release
+					m_dcf1.env.note_off_fast(&pv->dcf1_env);
+					m_lfo1.env.note_off_fast(&pv->lfo1_env);
+					m_dca1.env.note_off_fast(&pv->dca1_env);
+					m_note1[pv->note1] = 0;
+					pv->note1 = -1;
+				}
 			}
 			// synth 2
 			if (on2) {
-				// waveform
-				pv->note2 = key;
-				pv->vel2 = synthv1_velocity(vel, *m_def2.velocity);
-				// balance
-				pv->dco2_bal.reset(m_dco2.balance);
-				// pressure/after-touch
-				pv->pre2 = 0.0f;
-				pv->dca2_pre.reset(m_def2.pressure, &m_ctl2.pressure, &pv->pre2);
-				// frequencies
-				const float note2 = float(key)
-					+ *m_dco2.octave * OCTAVE_SCALE
-					+ *m_dco2.tuning * TUNING_SCALE;
-				const float detune2
-					= *m_dco2.detune * DETUNE_SCALE;
-				pv->dco2_freq1 = synthv1_freq(note2 - detune2);
-				pv->dco2_freq2 = synthv1_freq(note2 + detune2);
-				// phases
-				const float phase2 = *m_dco2.phase * PHASE_SCALE;
-				pv->dco2_sample1 = pv->dco2_osc1.start(  0.0f, pv->dco2_freq1);
-				pv->dco2_sample2 = pv->dco2_osc2.start(phase2, pv->dco2_freq2);
-				// filters
-				const int type2 = int(*m_dcf2.type);
-				pv->dcf21.reset(synthv1_filter1::Type(type2));
-				pv->dcf22.reset(synthv1_filter1::Type(type2));
-				pv->dcf23.reset(synthv1_filter2::Type(type2));
-				pv->dcf24.reset(synthv1_filter2::Type(type2));
-				// envelopes
-				m_dcf2.env.start(&pv->dcf2_env);
-				m_lfo2.env.start(&pv->lfo2_env);
-				m_dca2.env.start(&pv->dca2_env);
-				// lfos
-				pv->lfo2_sample = pv->lfo2_osc.start();
-				// glides (portamento)
-				const float frames2
-					= uint32_t(*m_dco2.glide * *m_dco2.glide * srate);
-				pv->dco2_glide2.reset(frames2, pv->dco2_freq2);
-				// sustain
-				pv->sustain2 = false;
-				// allocated
-				m_note2[key] = pv;
-			}
-		}
-	}
-	// note off
-	else if (status == 0x80 || (status == 0x90 && value == 0)) {
-		synthv1_voice *pv;
-		// synth 1
-		if (on1) {
-			pv = m_note1[key];
-			if (pv && pv->note1 >= 0) {
-				if (m_ctl1.sustain) {
-					pv->sustain1 = true;
-				} else {
-					if (pv->dca1_env.stage != synthv1_env::Release) {
-						m_dca1.env.note_off(&pv->dca1_env);
-						m_dcf1.env.note_off(&pv->dcf1_env);
-						m_lfo1.env.note_off(&pv->lfo1_env);
+				// mono voice modes
+				if (*m_def2.mono > 0.0f) {
+					for (pv = m_play_list.next(); pv; pv = pv->next()) {
+						if (pv->note2 >= 0
+							&& pv->dca2_env.stage != synthv1_env::Release) {
+							m_dcf2.env.note_off_fast(&pv->dcf2_env);
+							m_lfo2.env.note_off_fast(&pv->lfo2_env);
+							m_dca2.env.note_off_fast(&pv->dca2_env);
+							m_note2[pv->note2] = 0;
+							pv->note2 = -1;
+						}
 					}
 				}
+				// note retrigger
+				pv = m_note2[key];
+				if (pv && pv->note2 >= 0/* && !m_ctl2.sustain*/) {
+					// retrigger fast release
+					m_dcf2.env.note_off_fast(&pv->dcf2_env);
+					m_lfo2.env.note_off_fast(&pv->lfo2_env);
+					m_dca2.env.note_off_fast(&pv->dca2_env);
+					m_note2[pv->note2] = 0;
+					pv->note2 = -1;
+				}
+			}
+			// find free voice
+			pv = alloc_voice();
+			if (pv) {
+				// velocity (quadratic velocity law)
+				float vel = float(value) / 127.0f; vel *= vel;
+				// synth 1
+				if (on1) {
+					// waveform
+					pv->note1 = key;
+					pv->vel1 = synthv1_velocity(vel, *m_def1.velocity);
+					// balance
+					pv->dco1_bal.reset(m_dco1.balance);
+					// pressure/after-touch
+					pv->pre1 = 0.0f;
+					pv->dca1_pre.reset(m_def1.pressure, &m_ctl1.pressure, &pv->pre1);
+					// frequencies
+					const float note1 = float(key)
+						+ *m_dco1.octave * OCTAVE_SCALE
+						+ *m_dco1.tuning * TUNING_SCALE;
+					const float detune1
+						= *m_dco1.detune * DETUNE_SCALE;
+					pv->dco1_freq1 = synthv1_freq(note1 - detune1);
+					pv->dco1_freq2 = synthv1_freq(note1 + detune1);
+					// phases
+					const float phase1 = *m_dco1.phase * PHASE_SCALE;
+					pv->dco1_sample1 = pv->dco1_osc1.start(  0.0f, pv->dco1_freq1);
+					pv->dco1_sample2 = pv->dco1_osc2.start(phase1, pv->dco1_freq2);
+					// filters
+					const int type1 = int(*m_dcf1.type);
+					pv->dcf11.reset(synthv1_filter1::Type(type1));
+					pv->dcf12.reset(synthv1_filter1::Type(type1));
+					pv->dcf13.reset(synthv1_filter2::Type(type1));
+					pv->dcf14.reset(synthv1_filter2::Type(type1));
+					// envelopes
+					m_dcf1.env.start(&pv->dcf1_env);
+					m_lfo1.env.start(&pv->lfo1_env);
+					m_dca1.env.start(&pv->dca1_env);
+					// lfos
+					pv->lfo1_sample = pv->lfo1_osc.start();
+					// glides (portamento)
+					const float frames1
+						= uint32_t(*m_dco1.glide * *m_dco1.glide * m_srate);
+					pv->dco1_glide1.reset(frames1, pv->dco1_freq1);
+					pv->dco1_glide2.reset(frames1, pv->dco1_freq2);
+					// sustain
+					pv->sustain1 = false;
+					// allocated
+					m_note1[key] = pv;
+				}
+				// synth 2
+				if (on2) {
+					// waveform
+					pv->note2 = key;
+					pv->vel2 = synthv1_velocity(vel, *m_def2.velocity);
+					// balance
+					pv->dco2_bal.reset(m_dco2.balance);
+					// pressure/after-touch
+					pv->pre2 = 0.0f;
+					pv->dca2_pre.reset(m_def2.pressure, &m_ctl2.pressure, &pv->pre2);
+					// frequencies
+					const float note2 = float(key)
+						+ *m_dco2.octave * OCTAVE_SCALE
+						+ *m_dco2.tuning * TUNING_SCALE;
+					const float detune2
+						= *m_dco2.detune * DETUNE_SCALE;
+					pv->dco2_freq1 = synthv1_freq(note2 - detune2);
+					pv->dco2_freq2 = synthv1_freq(note2 + detune2);
+					// phases
+					const float phase2 = *m_dco2.phase * PHASE_SCALE;
+					pv->dco2_sample1 = pv->dco2_osc1.start(  0.0f, pv->dco2_freq1);
+					pv->dco2_sample2 = pv->dco2_osc2.start(phase2, pv->dco2_freq2);
+					// filters
+					const int type2 = int(*m_dcf2.type);
+					pv->dcf21.reset(synthv1_filter1::Type(type2));
+					pv->dcf22.reset(synthv1_filter1::Type(type2));
+					pv->dcf23.reset(synthv1_filter2::Type(type2));
+					pv->dcf24.reset(synthv1_filter2::Type(type2));
+					// envelopes
+					m_dcf2.env.start(&pv->dcf2_env);
+					m_lfo2.env.start(&pv->lfo2_env);
+					m_dca2.env.start(&pv->dca2_env);
+					// lfos
+					pv->lfo2_sample = pv->lfo2_osc.start();
+					// glides (portamento)
+					const float frames2
+						= uint32_t(*m_dco2.glide * *m_dco2.glide * m_srate);
+					pv->dco2_glide1.reset(frames2, pv->dco2_freq1);
+					pv->dco2_glide2.reset(frames2, pv->dco2_freq2);
+					// sustain
+					pv->sustain2 = false;
+					// allocated
+					m_note2[key] = pv;
+				}
 			}
 		}
-		// synth 2
-		if (on2) {
-			pv = m_note2[key];
-			if (pv && pv->note2 >= 0) {
-				if (m_ctl2.sustain) {
-					pv->sustain2 = true;
-				} else {
-					if (pv->dca2_env.stage != synthv1_env::Release) {
-						m_dca2.env.note_off(&pv->dca2_env);
-						m_dcf2.env.note_off(&pv->dcf2_env);
-						m_lfo2.env.note_off(&pv->lfo2_env);
-					}
-				}
-			}
-		}
-	}
-	// key pressure/poly.aftertouch
-	else if (status == 0xa0) {
-		const float pre = float(value) / 127.0f;
-		synthv1_voice *pv;
-		// synth 1
-		if (on1) {
-			pv = m_note1[key];
-			if (pv && pv->note1 >= 0)
-				pv->pre1 = *m_def1.pressure * pre;
-		}
-		// synth 2
-		if (on2) {
-			pv = m_note2[key];
-			if (pv && pv->note2 >= 0)
-				pv->pre2 = *m_def2.pressure * pre;
-		}
-	}
-	// control change
-	else if (status == 0xb0) {
-		switch (key) {
-		case 0x00:
-			// bank-select MSB (cc#0)
-			m_programs.bank_select_msb(value);
-			break;
-		case 0x01: {
-			// modulation wheel (cc#1)
-			const float mod = float(value) / 127.0f;
-			if (on1) m_ctl1.modwheel = *m_def1.modwheel * mod;
-			if (on2) m_ctl2.modwheel = *m_def2.modwheel * mod;
-			break;
-		}
-		case 0x07: {
-			// channel volume (cc#7)
-			const float vol = float(value) / 127.0f;
-			if (on1) m_ctl1.volume = vol;
-			if (on2) m_ctl2.volume = vol;
-			break;
-		}
-		case 0x0a: {
-			// channel panning (cc#10)
-			const float pan = float(value - 64) / 64.0f;
-			if (on1) m_ctl1.panning = pan;
-			if (on2) m_ctl2.panning = pan;
-			break;
-		}
-		case 0x20:
-			// bank-select LSB (cc#32)
-			m_programs.bank_select_lsb(value);
-			break;
-		case 0x40:
-			// sustain/damper pedal (cc#64)
+		// note off
+		else if (status == 0x80 || (status == 0x90 && value == 0)) {
+			synthv1_voice *pv;
+			// synth 1
 			if (on1) {
-				if (m_ctl1.sustain && value <  64)
-					allSustainOff_1();
-				m_ctl1.sustain = bool(value >= 64);
+				pv = m_note1[key];
+				if (pv && pv->note1 >= 0) {
+					if (m_ctl1.sustain) {
+						pv->sustain1 = true;
+					} else {
+						if (pv->dca1_env.stage != synthv1_env::Release) {
+							m_dca1.env.note_off(&pv->dca1_env);
+							m_dcf1.env.note_off(&pv->dcf1_env);
+							m_lfo1.env.note_off(&pv->lfo1_env);
+						}
+					}
+				}
 			}
+			// synth 2
 			if (on2) {
-				if (m_ctl2.sustain && value <  64)
-					allSustainOff_2();
-				m_ctl2.sustain = bool(value >= 64);
+				pv = m_note2[key];
+				if (pv && pv->note2 >= 0) {
+					if (m_ctl2.sustain) {
+						pv->sustain2 = true;
+					} else {
+						if (pv->dca2_env.stage != synthv1_env::Release) {
+							m_dca2.env.note_off(&pv->dca2_env);
+							m_dcf2.env.note_off(&pv->dcf2_env);
+							m_lfo2.env.note_off(&pv->lfo2_env);
+						}
+					}
+				}
 			}
-			break;
-		case 0x78:
-			// all sound off (cc#120)
-			allSoundOff();
-			break;
-		case 0x79:
-			// all controllers off (cc#121)
-			if (on1) allControllersOff_1();
-			if (on2) allControllersOff_2();
-			break;
-		case 0x7b:
-			// all notes off (cc#123)
-			if (on1) allNotesOff_1();
-			if (on2) allNotesOff_2();
-			break;
 		}
+		// key pressure/poly.aftertouch
+		else if (status == 0xa0) {
+			const float pre = float(value) / 127.0f;
+			synthv1_voice *pv;
+			// synth 1
+			if (on1) {
+				pv = m_note1[key];
+				if (pv && pv->note1 >= 0)
+					pv->pre1 = *m_def1.pressure * pre;
+			}
+			// synth 2
+			if (on2) {
+				pv = m_note2[key];
+				if (pv && pv->note2 >= 0)
+					pv->pre2 = *m_def2.pressure * pre;
+			}
+		}
+		// control change
+		else if (status == 0xb0) {
+			switch (key) {
+			case 0x00:
+				// bank-select MSB (cc#0)
+				m_programs.bank_select_msb(value);
+				break;
+			case 0x01: {
+				// modulation wheel (cc#1)
+				const float mod = float(value) / 127.0f;
+				if (on1) m_ctl1.modwheel = *m_def1.modwheel * mod;
+				if (on2) m_ctl2.modwheel = *m_def2.modwheel * mod;
+				break;
+			}
+			case 0x07: {
+				// channel volume (cc#7)
+				const float vol = float(value) / 127.0f;
+				if (on1) m_ctl1.volume = vol;
+				if (on2) m_ctl2.volume = vol;
+				break;
+			}
+			case 0x0a: {
+				// channel panning (cc#10)
+				const float pan = float(value - 64) / 64.0f;
+				if (on1) m_ctl1.panning = pan;
+				if (on2) m_ctl2.panning = pan;
+				break;
+			}
+			case 0x20:
+				// bank-select LSB (cc#32)
+				m_programs.bank_select_lsb(value);
+				break;
+			case 0x40:
+				// sustain/damper pedal (cc#64)
+				if (on1) {
+					if (m_ctl1.sustain && value <  64)
+						allSustainOff_1();
+					m_ctl1.sustain = bool(value >= 64);
+				}
+				if (on2) {
+					if (m_ctl2.sustain && value <  64)
+						allSustainOff_2();
+					m_ctl2.sustain = bool(value >= 64);
+				}
+				break;
+			case 0x78:
+				// all sound off (cc#120)
+				allSoundOff();
+				break;
+			case 0x79:
+				// all controllers off (cc#121)
+				if (on1) allControllersOff_1();
+				if (on2) allControllersOff_2();
+				break;
+			case 0x7b:
+				// all notes off (cc#123)
+				if (on1) allNotesOff_1();
+				if (on2) allNotesOff_2();
+				break;
+			}
+			// process controllers...
+			m_controls.process_enqueue(channel, key, value);
+		}
+		// pitch bend
+		else if (status == 0xe0) {
+			const float pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
+			if (on1) m_ctl1.pitchbend = synthv1_pow2f(*m_def1.pitchbend * pitchbend);
+			if (on2) m_ctl2.pitchbend = synthv1_pow2f(*m_def2.pitchbend * pitchbend);
+		}
+
 	}
-	// pitch bend
-	else if (status == 0xe0) {
-		const float pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
-		if (on1) m_ctl1.pitchbend = synthv1_pow2f(*m_def1.pitchbend * pitchbend);
-		if (on2) m_ctl2.pitchbend = synthv1_pow2f(*m_def2.pitchbend * pitchbend);
-	}
+
+	// process pending controllers...
+	m_controls.process_dequeue();
 }
 
 
@@ -1815,24 +1882,32 @@ void synthv1_impl::allSustainOff_2 (void)
 
 void synthv1_impl::allSoundOff (void)
 {
-	m_chorus.setSampleRate(m_iSampleRate);
+	m_chorus.setSampleRate(m_srate);
 	m_chorus.reset();
 
-	for (uint16_t k = 0; k < m_iChannels; ++k) {
-		m_phaser[k].setSampleRate(m_iSampleRate);
-		m_delay[k].setSampleRate(m_iSampleRate);
-		m_comp[k].setSampleRate(m_iSampleRate);
+	for (uint16_t k = 0; k < m_nchannels; ++k) {
+		m_phaser[k].setSampleRate(m_srate);
+		m_delay[k].setSampleRate(m_srate);
+		m_comp[k].setSampleRate(m_srate);
 		m_flanger[k].reset();
 		m_phaser[k].reset();
 		m_delay[k].reset();
 		m_comp[k].reset();
 	}
 
-	m_reverb.setSampleRate(m_iSampleRate);
+	m_reverb.setSampleRate(m_srate);
 	m_reverb.reset();
 }
 
 
+// controllers accessor
+
+synthv1_controls *synthv1_impl::controls (void)
+{
+	return &m_controls;
+}
+
+
 // programs accessor
 
 synthv1_programs *synthv1_impl::programs (void)
@@ -1845,7 +1920,7 @@ synthv1_programs *synthv1_impl::programs (void)
 
 void synthv1_impl::reset (void)
 {
-#if 1//--legacy support < 0.3.0.4
+#if 0//--legacy support < 0.3.0.4
 	if (*m_del.bpm < 3.6f)
 		*m_del.bpm *= 100.0f;
 #endif
@@ -1854,7 +1929,7 @@ void synthv1_impl::reset (void)
 	m_del.bpmsync0 = 0.0f;
 	m_del.bpm0 = m_del.bpm;
 
-m_vol1.reset(m_out1.volume, m_dca1.volume, &m_ctl1.volume, &m_aux1.volume);
+	m_vol1.reset(m_out1.volume, m_dca1.volume, &m_ctl1.volume, &m_aux1.volume);
 	m_pan1.reset(m_out1.panning, &m_ctl1.panning, &m_aux1.panning);
 	m_wid1.reset(m_out1.width);
 
@@ -1864,23 +1939,26 @@ m_vol1.reset(m_out1.volume, m_dca1.volume, &m_ctl1.volume, &m_aux1.volume);
 	
 	// flangers
 	if (m_flanger == 0)
-		m_flanger = new synthv1_fx_flanger [m_iChannels];
+		m_flanger = new synthv1_fx_flanger [m_nchannels];
 
 	// phasers
 	if (m_phaser == 0)
-		m_phaser = new synthv1_fx_phaser [m_iChannels];
+		m_phaser = new synthv1_fx_phaser [m_nchannels];
 
 	// delays
 	if (m_delay == 0)
-		m_delay = new synthv1_fx_delay [m_iChannels];
+		m_delay = new synthv1_fx_delay [m_nchannels];
 
 	// compressors
 	if (m_comp == 0)
-		m_comp = new synthv1_fx_comp [m_iChannels];
+		m_comp = new synthv1_fx_comp [m_nchannels];
 
 	// reverbs
 	m_reverb.reset();
 
+	// controllers reset.
+	m_controls.reset();
+
 	allSoundOff();
 //	allControllersOff();
 	allNotesOff();
@@ -1891,13 +1969,13 @@ m_vol1.reset(m_out1.volume, m_dca1.volume, &m_ctl1.volume, &m_aux1.volume);
 
 void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 {
-	float *v_outs[m_iChannels];
+	float *v_outs[m_nchannels];
 
 	// buffer i/o transfer
 
 	uint16_t k;
 
-	for (k = 0; k < m_iChannels; ++k)
+	for (k = 0; k < m_nchannels; ++k)
 		::memcpy(outs[k], ins[k], nframes * sizeof(float));
 
 	// controls
@@ -1951,7 +2029,7 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 		// output buffers
 
-		for (k = 0; k < m_iChannels; ++k)
+		for (k = 0; k < m_nchannels; ++k)
 			v_outs[k] = outs[k];
 
 		uint32_t nblock = nframes;
@@ -2071,7 +2149,7 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 					= vol1 * (mid1 - sid1 * wid1) * m_pan1.value(j, 1)
 					+ vol2 * (mid2 - sid2 * wid2) * m_pan2.value(j, 1);
 
-				for (k = 0; k < m_iChannels; ++k)
+				for (k = 0; k < m_nchannels; ++k)
 					*v_outs[k]++ += (k & 1 ? out2 : out1);
 
 				if (j == 0) {
@@ -2131,13 +2209,13 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	}
 
 	// chorus
-	if (m_iChannels > 1) {
+	if (m_nchannels > 1) {
 		m_chorus.process(outs[0], outs[1], nframes, *m_cho.wet,
 			*m_cho.delay, *m_cho.feedb, *m_cho.rate, *m_cho.mod);
 	}
 
 	// effects
-	for (k = 0; k < m_iChannels; ++k) {
+	for (k = 0; k < m_nchannels; ++k) {
 		float *in = outs[k];
 		// flanger
 		m_flanger[k].process(in, nframes, *m_fla.wet,
@@ -2151,13 +2229,13 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	}
 
 	// reverb
-	if (m_iChannels > 1) {
+	if (m_nchannels > 1) {
 		m_reverb.process(outs[0], outs[1], nframes, *m_rev.wet,
 			*m_rev.feedb, *m_rev.room, *m_rev.damp, *m_rev.width);
 	}
 
 	// dynamics
-	for (k = 0; k < m_iChannels; ++k) {
+	for (k = 0; k < m_nchannels; ++k) {
 		float *in = outs[k];
 		// compressor
 		if (int(*m_dyn.compress) > 0)
@@ -2179,6 +2257,8 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	m_wid2.process(nframes);
 	m_pan2.process(nframes);
 	m_vol2.process(nframes);
+
+	m_controls.process(nframes);
 }
 
 
@@ -2187,9 +2267,9 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 // synthv1 - decl.
 //
 
-synthv1::synthv1 ( uint16_t iChannels, uint32_t iSampleRate )
+synthv1::synthv1 ( uint16_t nchannels, float srate )
 {
-	m_pImpl = new synthv1_impl(this, iChannels, iSampleRate);
+	m_pImpl = new synthv1_impl(this, nchannels, srate);
 }
 
 
@@ -2199,9 +2279,9 @@ synthv1::~synthv1 (void)
 }
 
 
-void synthv1::setChannels ( uint16_t iChannels )
+void synthv1::setChannels ( uint16_t nchannels )
 {
-	m_pImpl->setChannels(iChannels);
+	m_pImpl->setChannels(nchannels);
 }
 
 
@@ -2211,13 +2291,13 @@ uint16_t synthv1::channels (void) const
 }
 
 
-void synthv1::setSampleRate ( uint32_t iSampleRate )
+void synthv1::setSampleRate ( float srate )
 {
-	m_pImpl->setSampleRate(iSampleRate);
+	m_pImpl->setSampleRate(srate);
 }
 
 
-uint32_t synthv1::sampleRate (void) const
+float synthv1::sampleRate (void) const
 {
 	return m_pImpl->sampleRate();
 }
@@ -2264,6 +2344,14 @@ void synthv1::process ( float **ins, float **outs, uint32_t nframes )
 }
 
 
+// controllers accessor
+
+synthv1_controls *synthv1::controls (void) const
+{
+	return m_pImpl->controls();
+}
+
+
 // programs accessor
 
 synthv1_programs *synthv1::programs (void) const
