@@ -28,6 +28,7 @@
 
 // forward declarations
 class synthv1_impl;
+class synthv1_controls;
 class synthv1_programs;
 
 
@@ -39,15 +40,15 @@ class synthv1
 {
 public:
 
-	synthv1(uint16_t iChannels = 2, uint32_t iSampleRate = 44100);
+	synthv1(uint16_t nchannels = 2, float srate = 44100.0f);
 
 	~synthv1();
 
-	void setChannels(uint16_t iChannels);
+	void setChannels(uint16_t nchannels);
 	uint16_t channels() const;
 
-	void setSampleRate(uint32_t iSampleRate);
-	uint32_t sampleRate() const;
+	void setSampleRate(float srate);
+	float sampleRate() const;
 
 	enum ParamIndex	 {
 
@@ -187,6 +188,7 @@ public:
 	void setParamPort(ParamIndex index, float *pfParam);
 	float *paramPort(ParamIndex index) const;
 
+	synthv1_controls *controls() const;
 	synthv1_programs *programs() const;
 
 	void setParamValue(ParamIndex index, float fValue);
