@@ -23,6 +23,9 @@
 #include "synthv1_config.h"
 #include "synthv1_param.h"
 
+#include "synthv1_programs.h"
+#include "synthv1_controls.h"
+
 #include <jack/midiport.h>
 
 #include <stdio.h>
@@ -164,6 +167,9 @@ synthv1_jack::synthv1_jack (void) : synthv1(2)
 
 	m_bpm = 0.0f;
 
+	synthv1::programs()->enabled(true);
+	synthv1::controls()->enabled(true);
+
 	open(SYNTHV1_TITLE);
 	activate();
 }
@@ -273,7 +279,7 @@ void synthv1_jack::open ( const char *client_id )
 {
 	// init param ports
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
-		synthv1::ParamIndex index = synthv1::ParamIndex(i);
+		const synthv1::ParamIndex index = synthv1::ParamIndex(i);
 		m_params[i] = synthv1_param::paramDefaultValue(index);
 		synthv1::setParamPort(index, &m_params[i]);
 	}
@@ -284,7 +290,7 @@ void synthv1_jack::open ( const char *client_id )
 		return;
 
 	// set sample rate
-	synthv1::setSampleRate(jack_get_sample_rate(m_client));
+	synthv1::setSampleRate(float(jack_get_sample_rate(m_client)));
 //	synthv1::reset();
 
 	// register audio ports & buffers
