@@ -26,6 +26,7 @@
 #include "synthv1_sched.h"
 
 #include "synthv1widget_config.h"
+#include "synthv1widget_control.h"
 
 #include <QMessageBox>
 #include <QDir>
@@ -63,7 +64,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.setupUi(this);
 
 	// Init sched notifier.
-	m_sched_notifier = new synthv1_sched_notifier(this);
+	m_sched_notifier = NULL;
 
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i)
@@ -731,11 +732,6 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
-	// Special sample update notifications (eg. reverse)
-	QObject::connect(m_sched_notifier,
-		SIGNAL(notify(int)),
-		SLOT(updateSchedNotify(int)));
-
 	// General knob/dial  behavior init...
 	synthv1_config *pConfig = synthv1_config::getInstance();
 	if (pConfig) {
@@ -752,6 +748,34 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 }
 
 
+// Destructor.
+synthv1widget::~synthv1widget (void)
+{
+	if (m_sched_notifier)
+		delete m_sched_notifier;
+}
+
+
+// Create/initialize the scheduler/work notifier.
+void synthv1widget::initSchedNotifier (void)
+{
+	if (m_sched_notifier) {
+		delete m_sched_notifier;
+		m_sched_notifier = NULL;
+	}
+
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi == NULL)
+		return;
+
+	m_sched_notifier = new synthv1widget_sched(pSynthUi->instance(), this);
+
+	QObject::connect(m_sched_notifier,
+		SIGNAL(notify(int, int)),
+		SLOT(updateSchedNotify(int, int)));
+}
+
+
 // Param kbob (widget) map accesors.
 void synthv1widget::setParamKnob ( synthv1::ParamIndex index, synthv1widget_knob *pKnob )
 {
@@ -763,6 +787,12 @@ void synthv1widget::setParamKnob ( synthv1::ParamIndex index, synthv1widget_knob
 	QObject::connect(pKnob,
 		SIGNAL(valueChanged(float)),
 		SLOT(paramChanged(float)));
+
+	pKnob->setContextMenuPolicy(Qt::CustomContextMenu);
+
+	QObject::connect(pKnob,
+		SIGNAL(customContextMenuRequested(const QPoint&)),
+		SLOT(paramContextMenu(const QPoint&)));
 }
 
 synthv1widget_knob *synthv1widget::paramKnob ( synthv1::ParamIndex index ) const
@@ -811,11 +841,10 @@ void synthv1widget::paramChanged ( float fValue )
 
 	synthv1widget_knob *pKnob = qobject_cast<synthv1widget_knob *> (sender());
 	if (pKnob) {
-		synthv1::ParamIndex index = m_knobParams.value(pKnob);
+		const synthv1::ParamIndex index = m_knobParams.value(pKnob);
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
-		m_ui.StatusBar->showMessage(QString("%1 / %2: %3")
-			.arg(m_ui.StackedWidget->currentWidget()->windowTitle())
+		m_ui.StatusBar->showMessage(QString("%1: %2")
 			.arg(pKnob->toolTip())
 			.arg(pKnob->valueText()), 5000);
 		updateDirtyPreset(true);
@@ -857,6 +886,26 @@ void synthv1widget::updateParamEx ( synthv1::ParamIndex index, float fValue )
 }
 
 
+// Update scheduled controllers param/knob widgets.
+void synthv1widget::updateSchedParam ( synthv1::ParamIndex index, float fValue )
+{
+	++m_iUpdate;
+
+	synthv1widget_knob *pKnob = paramKnob(index);
+	if (pKnob) {
+		pKnob->setValue(fValue, false);
+		updateParam(index, fValue);
+		updateParamEx(index, fValue);
+		m_ui.StatusBar->showMessage(QString("%1: %2")
+			.arg(pKnob->toolTip())
+			.arg(pKnob->valueText()), 5000);
+		updateDirtyPreset(true);
+	}
+
+	--m_iUpdate;
+}
+
+
 // Reset all param knobs to default values.
 void synthv1widget::resetParams (void)
 {
@@ -869,7 +918,7 @@ void synthv1widget::resetParams (void)
 	resetSwapParams();
 
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
-		synthv1::ParamIndex index = synthv1::ParamIndex(i);
+		const synthv1::ParamIndex index = synthv1::ParamIndex(i);
 		float fValue = synthv1_param::paramDefaultValue(index);
 		synthv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob)
@@ -896,7 +945,7 @@ void synthv1widget::swapParams ( bool bOn )
 //	resetParamKnobs();
 
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
-		synthv1::ParamIndex index = synthv1::ParamIndex(i);
+		const synthv1::ParamIndex index = synthv1::ParamIndex(i);
 		synthv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob) {
 			const float fOldValue = pKnob->value();
@@ -930,7 +979,7 @@ void synthv1widget::updateParamValues (void)
 	synthv1_ui *pSynthUi = ui_instance();
 
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
-		synthv1::ParamIndex index = synthv1::ParamIndex(i);
+		const synthv1::ParamIndex index = synthv1::ParamIndex(i);
 		const float fValue = (pSynthUi
 			? pSynthUi->paramValue(index)
 			: synthv1_param::paramDefaultValue(index));
@@ -948,7 +997,7 @@ void synthv1widget::resetParamValues (void)
 	resetSwapParams();
 
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
-		synthv1::ParamIndex index = synthv1::ParamIndex(i);
+		const synthv1::ParamIndex index = synthv1::ParamIndex(i);
 		const float fValue = synthv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
@@ -1052,17 +1101,31 @@ void synthv1widget::updateLoadPreset ( const QString& sPreset )
 
 
 // Notification updater.
-void synthv1widget::updateSchedNotify ( int stype )
+void synthv1widget::updateSchedNotify ( int stype, int sid )
 {
 	synthv1_ui *pSynthUi = ui_instance();
 	if (pSynthUi == NULL)
 		return;
 
 #ifdef CONFIG_DEBUG_0
-	qDebug("samplv1widget::updateSchedNotify(%d)", stype);
+	qDebug("samplv1widget::updateSchedNotify(%d, %d)", stype, sid);
 #endif
 
 	switch (synthv1_sched::Type(stype)) {
+	case synthv1_sched::Controller: {
+		synthv1widget_control *pInstance
+			= synthv1widget_control::getInstance();
+		if (pInstance) {
+			synthv1_controls *pControls = pSynthUi->controls();
+			pInstance->setControlKey(pControls->current_key());
+		}
+		break;
+	}
+	case synthv1_sched::Controls: {
+		const synthv1::ParamIndex index = synthv1::ParamIndex(sid);
+		updateSchedParam(index, pSynthUi->paramValue(index));
+		break;
+	}
 	case synthv1_sched::Programs: {
 		synthv1_programs *pPrograms = pSynthUi->programs();
 		synthv1_programs::Prog *pProg = pPrograms->current_prog();
@@ -1085,8 +1148,10 @@ void synthv1widget::helpConfigure (void)
 
 	synthv1widget_config form(this);
 
-	// Set programs database...
+	// Set controllers&&programs database...
+	form.setControls(pSynthUi->controls());
 	form.setPrograms(pSynthUi->programs());
+
 	form.exec();
 }
 
@@ -1173,4 +1238,37 @@ void synthv1widget::bpmSyncChanged (void)
 }
 
 
+// Param knob context menu.
+void synthv1widget::paramContextMenu ( const QPoint& pos )
+{
+	synthv1widget_knob *pKnob
+		= qobject_cast<synthv1widget_knob *> (sender());
+	if (pKnob == NULL)
+		return;
+
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi == NULL)
+		return;
+
+	synthv1_controls *pControls = pSynthUi->controls();
+	if (pControls == NULL)
+		return;
+
+	if (!pControls->enabled())
+		return;
+
+	QMenu menu(this);
+
+	QAction *pAction = menu.addAction(
+		QIcon(":/images/synthv1_control.png"),
+		tr("MIDI &Controller..."));
+
+	if (menu.exec(pKnob->mapToGlobal(pos)) == pAction) {
+		const synthv1::ParamIndex index = m_knobParams.value(pKnob);
+		const QString& sTitle = pKnob->toolTip();
+		synthv1widget_control::showInstance(pControls, index, sTitle, this);
+	}
+}
+
+
 // end of synthv1widget.cpp
