@@ -72,6 +72,9 @@ synthv1widget_jack::synthv1widget_jack ( synthv1_jack *pSynth )
 	// Initialize (user) interface stuff...
 	m_pSynthUi = new synthv1_ui(m_pSynth);
 
+	// May initialize the scheduler/work notifier.
+	initSchedNotifier();
+
 	// Initialize preset stuff...
 	// initPreset();
 	updateParamValues();
