@@ -47,6 +47,9 @@ synthv1widget_lv2::synthv1widget_lv2 ( synthv1_lv2 *pSynth,
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i)
 		m_params_def[i] = true;
 
+	// May initialize the scheduler/work notifier.
+	initSchedNotifier();
+
 	clearPreset();
 }
 
@@ -116,13 +119,13 @@ void synthv1widget_lv2::port_event ( uint32_t port_index,
 	uint32_t buffer_size, uint32_t format, const void *buffer )
 {
 	if (format == 0 && buffer_size == sizeof(float)) {
-		synthv1::ParamIndex index
+		const synthv1::ParamIndex index
 			= synthv1::ParamIndex(port_index - synthv1_lv2::ParamBase);
-		float fValue = *(float *) buffer;
-	//--legacy support < 0.3.0.4 -- begin
+		const float fValue = *(float *) buffer;
+	#if 0//--legacy support < 0.3.0.4 -- begin
 		if (index == synthv1::DEL1_BPM && fValue < 3.6f)
 			fValue *= 100.0f;
-	//--legacy support < 0.3.0.4 -- end.
+	#endif
 		setParamValue(index, fValue, m_params_def[index]);
 		m_params_def[index] = false;
 	}
