@@ -1,7 +1,7 @@
 // synthv1widget_preset.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -21,7 +21,7 @@
 
 #include "synthv1widget_preset.h"
 
-#include "synthv1widget_config.h"
+#include "synthv1_config.h"
 
 #include <QHBoxLayout>
 
@@ -109,19 +109,12 @@ synthv1widget_preset::synthv1widget_preset ( QWidget *pParent )
 }
 
 
-// Preset group path name.
-QString synthv1widget_preset::presetGroup (void) const
-{
-	return "/Presets/";
-}
-
-
 // Preset name/text accessors.
 void synthv1widget_preset::clearPreset (void)
 {
 	++m_iInitPreset;
 
-	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
+	synthv1_config *pConfig = synthv1_config::getInstance();
 	if (pConfig)
 		pConfig->sPreset.clear();
 
@@ -133,6 +126,10 @@ void synthv1widget_preset::clearPreset (void)
 
 void synthv1widget_preset::setPreset ( const QString& sPreset )
 {
+	synthv1_config *pConfig = synthv1_config::getInstance();
+	if (pConfig)
+		pConfig->sPreset = sPreset;
+
 	const bool bBlockSignals = m_pComboBox->blockSignals(true);
 	m_pComboBox->setEditText(sPreset);
 	m_pComboBox->blockSignals(bBlockSignals);
@@ -150,12 +147,12 @@ bool synthv1widget_preset::queryPreset (void)
 	if (m_iInitPreset == 0)
 		return true;
 
-	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
+	synthv1_config *pConfig = synthv1_config::getInstance();
 	if (pConfig == NULL)
 		return false;
 
 	if (m_iDirtyPreset > 0) {
-		const QString& sPreset = pConfig->sPreset;
+		const QString sPreset(pConfig->sPreset);
 		if (sPreset.isEmpty()) {
 			if (QMessageBox::warning(this,
 				tr("Warning") + " - " SYNTHV1_TITLE,
@@ -202,13 +199,11 @@ void synthv1widget_preset::loadPreset ( const QString& sPreset )
 	if (sPreset.isEmpty())
 		return;
 
-	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
+	synthv1_config *pConfig = synthv1_config::getInstance();
 	if (pConfig) {
-		pConfig->beginGroup(presetGroup());
-		emit loadPresetFile(pConfig->value(sPreset).toString());
-		pConfig->endGroup();
+		emit loadPresetFile(pConfig->presetFile(sPreset));
 		++m_iInitPreset;
-		pConfig->sPreset = sPreset;
+	//	pConfig->sPreset = sPreset;
 	//	setPreset(sPreset);
 		refreshPreset();
 	}
@@ -231,7 +226,7 @@ void synthv1widget_preset::newPreset (void)
 
 void synthv1widget_preset::openPreset (void)
 {
-	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
+	synthv1_config *pConfig = synthv1_config::getInstance();
 	if (pConfig == NULL)
 		return;
 
@@ -264,12 +259,10 @@ void synthv1widget_preset::openPreset (void)
 		QFileInfo fi(sFilename);
 		if (fi.exists() && queryPreset()) {
 			const QString& sPreset = fi.completeBaseName();
-			pConfig->beginGroup(presetGroup());
-			pConfig->setValue(sPreset, sFilename);
-			pConfig->endGroup();
+			pConfig->setPresetFile(sPreset, sFilename);
 			emit loadPresetFile(sFilename);
 			++m_iInitPreset;
-			pConfig->sPreset = sPreset;
+		//	pConfig->sPreset = sPreset;
 			pConfig->sPresetDir = fi.absolutePath();
 			setPreset(sPreset);
 			refreshPreset();
@@ -290,7 +283,7 @@ void synthv1widget_preset::savePreset ( const QString& sPreset )
 	if (sPreset.isEmpty())
 		return;
 
-	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
+	synthv1_config *pConfig = synthv1_config::getInstance();
 	if (pConfig == NULL)
 		return;
 
@@ -337,9 +330,7 @@ void synthv1widget_preset::savePreset ( const QString& sPreset )
 		if (QFileInfo(sFilename).suffix() != sExt)
 			sFilename += '.' + sExt;
 		emit savePresetFile(sFilename);
-		pConfig->beginGroup(presetGroup());
-		pConfig->setValue(sPreset, sFilename);
-		pConfig->endGroup();
+		pConfig->setPresetFile(sPreset, sFilename);
 		++m_iInitPreset;
 		pConfig->sPreset = sPreset;
 		pConfig->sPresetDir = QFileInfo(sFilename).absolutePath();
@@ -356,7 +347,7 @@ void synthv1widget_preset::deletePreset (void)
 	if (sPreset.isEmpty())
 		return;
 
-	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
+	synthv1_config *pConfig = synthv1_config::getInstance();
 	if (pConfig == NULL)
 		return;
 
@@ -370,12 +361,7 @@ void synthv1widget_preset::deletePreset (void)
 		== QMessageBox::Cancel)
 		return;
 
-	pConfig->beginGroup(presetGroup());
-	const QString& sFilename = pConfig->value(sPreset).toString();
-	if (QFileInfo(sFilename).exists())
-		QFile(sFilename).remove();
-	pConfig->remove(sPreset);
-	pConfig->endGroup();
+	pConfig->removePreset(sPreset);
 
 	clearPreset();
 	refreshPreset();
@@ -409,18 +395,14 @@ void synthv1widget_preset::refreshPreset (void)
 	const QString sOldPreset = m_pComboBox->currentText();
 	const QIcon icon(":/images/synthv1_preset.png");
 	m_pComboBox->clear();
-	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
+	synthv1_config *pConfig = synthv1_config::getInstance();
 	if (pConfig) {
-		pConfig->beginGroup(presetGroup());
-		const QStringList& list = pConfig->childKeys();
-		QStringListIterator iter(list);
+		QStringListIterator iter(pConfig->presetList());
 		while (iter.hasNext()) {
 			const QString& sPreset = iter.next();
-			if (QFileInfo(pConfig->value(sPreset).toString()).exists())
-				m_pComboBox->addItem(icon, sPreset);
+			m_pComboBox->addItem(icon, sPreset);
 		}
 		m_pComboBox->model()->sort(0);
-		pConfig->endGroup();
 	}
 
 	const int iIndex = m_pComboBox->findText(sOldPreset);
@@ -438,7 +420,7 @@ void synthv1widget_preset::refreshPreset (void)
 // Preset control.
 void synthv1widget_preset::initPreset (void)
 {
-	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
+	synthv1_config *pConfig = synthv1_config::getInstance();
 	if (pConfig && !pConfig->sPreset.isEmpty())
 		loadPreset(pConfig->sPreset);
 	else
